library provider_mvc;

import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

/// Controller base class
abstract class Controller extends ChangeNotifier {
  /// Set of controller objects
  static final Map _controllers = {};

  /// Optional controller tag
  final String tag;

  /// Constructor
  Controller({this.tag});

  /// Find controllers of some type by tag
  static Iterable<T> find<T extends Controller>(String tag) =>
      of<T>().where((element) => element.tag == tag);

  /// Get all registred controllers of some type
  static Set<T> of<T extends Controller>() => _controllers[T];

  /// Register a controller
  static T put<T extends Controller>(T controller) {
    if (_controllers[T] == null) _controllers[T] = Set<T>();
    _controllers[T].add(controller);
    return controller;
  }

  /// Refresh the view associated to the controller
  void refresh() => notifyListeners();
}

/// View base class
abstract class View<T extends Controller> extends StatelessWidget {
  /// Returns an instance of the controller associated to view
  T create(BuildContext context);

  /// Returns the view widget
  Widget builder(BuildContext context, T controller, Widget widget);

  /// View widget build function - not to override
  @override
  Widget build(BuildContext context) => ChangeNotifierProvider<T>(
        create: create,
        child: Consumer<T>(builder: builder),
      );
}
